<?php
namespace  Belgian_Tax;
/*
 *   This file is part of NOALYSS.
 *
 *   NOALYSS is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   NOALYSS is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with NOALYSS; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
/* $Revision$ */

// Copyright (c) 2002 Author Dany De Bontridder dany@alchimerys.be

/*!\file
 * \brief Compute VAT_AMOUNT
 */


/*!
 * \brief Compute VAT_AMOUNT
 */
#[\AllowDynamicProperties]
class Tva_Amount
{
    var $param ; //!< Code from TVA_BELGE.PARAMETER_CHLD
    var $detail_id ; //!< id for the details, saved into TVA_BELGE.DECLARATION_AMOUNT_DETAIL (detail_id)

  private static $variable=array('amount'=>'amount',
				 'amount_tva'=>'amount_tva',
				 'param'=>'param',
				 'start_periode'=>'start_periode',
				 'end_periode'=>'end_periode',
				 'grid'=>'grid'
				 );
  function __construct ($p_init,$p_start_periode,$p_end_periode) {
    $this->db=$p_init;
    $this->start_periode=$p_start_periode;
    $this->end_periode=$p_end_periode;
    $this->param=null;

  }
  public function get_parameter($p_string) {
    if ( array_key_exists($p_string,self::$variable) ) {
      $idx=self::$variable[$p_string];
      return $this->$idx;
    }
    else
      throw new \Exception("Attribut inexistant $p_string");
  }
  public function set_parameter($p_string,$p_value) {
    if ( array_key_exists($p_string,self::$variable) ) {
      $idx=self::$variable[$p_string];
      $this->$idx=$p_value;
    }
    else
      throw new \Exception("Attribut inexistant $p_string");


  }
  public function __toString() {    return var_export(self::$variable,true);  }
  public function verify() {
    // Verify that the elt we want to add is correct
  }
  /**
   *@brief load all tva_belge.parameter_chld (parameters detail) into $this->param
   */
  public function load_parameter() {
    if ( $this->param != null ) return;

    //  get the vat code
     $ctva=$this->db->get_array("
    select pi_id,
           tva_id,
           pcm_val,
           ledger_id,
           filter_ledger,
           p_operation ,
           p_side
    from 
        tva_belge.parameter_chld 
    where 
        pcode=$1
        ",array($this->grid));

     $this->param=$ctva;
  }
  /**
   *@brief get the amount of vat thanks its code, or special GRID 61 & 62,
   *  we can compute from MISC ledgers and   * not from quant_X tables
   *@param $p_gril \Belgian_Tax\is the gril code
      * out for the table quant_sold
   *@return \Belgian_Tax\a number
   */
  function amount_operation() {
        // load all parameter from PARAMETER_CHLD
        $this->load_parameter();
        $result = 0;
        bcscale(4);
        for ($i = 0; $i < count($this->param); $i++) {
            // for filter_ledger == MISC LEDGER special GRID 61 & 62,
            if ($this->param[$i]['filter_ledger'] == FILTER_LEDGER_MISC
               || $this->param[$i]['filter_ledger'] == FILTER_LEDGER_FINANCIAL
            ) {
                $acc_accounting = new \Acc_Account_Ledger($this->db,0);
                $acc_accounting->id = $this->param[$i]['pcm_val'];
                $jrn_type= match (intval($this->param[$i]['filter_ledger']))
                    {
                    FILTER_LEDGER_FINANCIAL=>'FIN',
                    FILTER_LEDGER_MISC=>'ODS'
                    };
                $cond = sprintf(" 
                j_jrn_def in (select jrn_def_id from jrn_def where jrn_def_type='%s')
            and  j_date >= to_date('%s','DD.MM.YYYY')
            and j_date <= to_date('%s','DD.MM.YYYY')",
                        $jrn_type,
                        $this->start_periode,
                        $this->end_periode
                );

                // $tmp_calc : float result for the amount
                $tmp_calc =$acc_accounting->get_solde($cond);
            } else {

                // $tmp_calc : float result for the amount
                $tmp_calc = $this->get_amount_filter($this->param[$i]);
            }
            $warning = "";
            if ($this->param[$i]['p_side'] == 'P' & $tmp_calc < 0)
                $warning = _('Montant négatif');
            if ($this->param[$i]['p_side'] == 'N' & $tmp_calc > 0)
                $warning = _('Montant positif');
            // p_operation == A for ADD
            if ($this->param[$i]['p_operation'] == 'A') {

                $result = bcadd($result, $tmp_calc);
                // p_operation == S for SUBSTRACT
            } elseif ($this->param[$i]['p_operation'] == 'S') {

                $result = bcsub($result, $tmp_calc);
            } else {
                throw new \Exception("TVA115 : error p_operation unknown {$this->param[$i]['p_operation']}");
            }
            // save only if different from 0
            if ($tmp_calc != 0)
                $this->db->exec_sql('insert into TVA_BELGE.DECLARATION_AMOUNT_DETAIL(detail_id,parameter_child_id,d1_amount,d1_info)
values ($1,$2,$3,$4)', [$this->detail_id, $this->param[$i]['pi_id'], $tmp_calc, $warning]);
        }
        return round($result, 2);
    }

    /**
     * @brief compute the amount of VAT of each row of tva_belge.parameter
     * @return float
     * @throws \Exception p_operation is not ADD or SUBSTRACT
     */
  function amount_vat(bool $autoreverse) {
    // get the VAT code
   $this->load_parameter();
    $result=0;
	bcscale(4);
	for ($i=0;$i<count($this->param);$i++)
	{
        if ( ! $autoreverse)
		    $tmp_calc=$this->get_vat_filter($this->param[$i]);
        else
            $tmp_calc=$this->get_vat_filter_autoreverse($this->param[$i]);

        $warning="";
        if ($this->param[$i]['p_side'] == 'P' & $tmp_calc < 0) $warning=_('Montant négatif');
        if ($this->param[$i]['p_side'] == 'N' & $tmp_calc > 0) $warning=_('Montant positif');
        if ($this->param[$i]['p_operation']=='A') {
            $result=bcadd($result,$tmp_calc);

        } elseif ($this->param[$i]['p_operation']=='S') {

            $result=bcsub($result,$tmp_calc);
        }else {
            throw new \Exception("TVA115 : error p_operation unknown {$this->param[$i]['p_operation']}");
        }
        // save only if different from 0
        if ($tmp_calc != 0 )

            $this->db->exec_sql('insert into TVA_BELGE.DECLARATION_AMOUNT_DETAIL(detail_id,parameter_child_id,d1_amount,d1_info)
values ($1,$2,$3,$4)',[$this->detail_id,$this->param[$i]['pi_id'],$tmp_calc,$warning]);

	}
    return round($result,2);


  }

  /**
   *@brief get the amount of operation from the table quant_sold or quant_purchase
   * @param $aParameter_chld array parameter (type row tva_belge.parameter_child)
   * @see \Belgian\DB\Parameter_Chld_SQL
   *@return float : amount
   */

  private function get_amount_filter($aParameter_chld) {

      if (trim($aParameter_chld['tva_id'])=="") return 0;

      // $pcm_val string to filter on accounting
      $pcm_val = (trim($aParameter_chld['pcm_val']??"") == "") ? "%" : $aParameter_chld['pcm_val'];

      // $a_sql_param array for SQL
      $a_sql_param=array(
          $aParameter_chld['tva_id'],
          $this->start_periode,
          $this->end_periode,
          $aParameter_chld['pcm_val']
          );

      // $filter_ledger string to filter on a specific ledger
      $filter_ledger="";
      if ( $aParameter_chld['filter_ledger']==3 && $aParameter_chld['ledger_id'] !="" ) {
          $filter_ledger=' and j_jrn_def=$5';
          $a_sql_param[]=$aParameter_chld['ledger_id'];
      }

    // from sales
    if ( $aParameter_chld['filter_ledger']==1 ) {
      $sql="select coalesce(sum(qs_price),0) as amount 
                from quant_sold
		        join jrnx using (j_id)
                where
                        qs_vat_code=$1
                    and (j_date >= to_date($2,'DD.MM.YYYY') 
                    and j_date <= to_date($3,'DD.MM.YYYY'))
                    and j_poste::text like ($4)
                    $filter_ledger
                    ";

      $res=$this->db->get_array($sql,$a_sql_param);
      return $res[0]['amount'];
    }
    // from purchase
    if (  $aParameter_chld['filter_ledger']==2   ) {
      $sql="select coalesce(sum(qp_price),0) as amount 
            from 
                quant_purchase 
                join jrnx using (j_id)
           where 
                   qp_vat_code=$1 
               and (j_date >= to_date($2,'DD.MM.YYYY') 
               and j_date <= to_date($3,'DD.MM.YYYY'))
               and j_poste::text like ($4)
               $filter_ledger
               ";
      $res=$this->db->get_array($sql,$a_sql_param);
      return $res[0]['amount'];
    }


    return 0;
  }

  /**
   *@brief get the amount of VAT from the table quant_sold or quant_purchase
   * @param $aParameter_chld array parameter (type row tva_belge.parameter_child)
   * @return float : amount
   * @see \Belgian\DB\Parameter_Chld_SQL
   */
    private function get_vat_filter($aParameter_chld)
    {
        if (trim($aParameter_chld['tva_id']) == "") return 0;

        // $pcm_val string to filter on accounting
        $pcm_val = (\noalyss_trim($aParameter_chld['pcm_val']) == "") ? "%" : $aParameter_chld['pcm_val'];

        // $a_sql_param array for SQL
        $a_sql_param = array(
            $aParameter_chld['tva_id'],
            $this->start_periode,
            $this->end_periode,
            $pcm_val
        );
        // variable: $flag_ledger to determine what formula to use to compute VAT, from PURCHASE or SALES
        $flag_ledger=$aParameter_chld['filter_ledger'] ;

        // variable: $filter_ledger string to filter on a specific ledger
        $filter_ledger = "";

        if ($aParameter_chld['filter_ledger'] == 3 && $aParameter_chld['ledger_id'] != "") {
            $filter_ledger = ' and j_jrn_def=$5';
            $a_sql_param[] = $aParameter_chld['ledger_id'];
            $x=$this->db->get_value('select jrn_def_type from jrn_def where jrn_def_id=$1',
                                                [$aParameter_chld['ledger_id']]);
            $flag_ledger=match($x){'VEN'=>1,'ACH'=>2,default=>0};

        }

        if ($flag_ledger == 0)
            throw new \Exception("TVMNT276. Journal invalid pour jrn_def_id{$aParameter_chld['ledger_id']}");

        // from sales
        if ($flag_ledger == 1) {
            $sql = "select coalesce(sum(qs_vat),0) as amount 
            from 
                quant_sold 
                join jrnx using (j_id)
           where 
               qs_vat_code=$1
                and (j_date >= to_date($2,'DD.MM.YYYY')
                and j_date <= to_date($3,'DD.MM.YYYY'))
                and j_poste::text like ($4)
                $filter_ledger
                ";
            $res = $this->db->get_array($sql, $a_sql_param);

            return $res[0]['amount'];
        }

        // from purchase
        if ($flag_ledger  == 2) {
            $sql = "select 
                coalesce(sum(qp_vat),0) as amount 
            from 
                quant_purchase 
                join jrnx using (j_id)
           where 
               qp_vat_code=$1
               and  (j_date >= to_date($2,'DD.MM.YYYY')
               and j_date <= to_date($3,'DD.MM.YYYY'))
               and j_poste::text like ($4)
               $filter_ledger
               ";
            $res = $this->db->get_array($sql, $a_sql_param);
            return $res[0]['amount'];
        }


        return 0;
    }
    /**
     *@brief get the amount of VAT from the table quant_sold or quant_purchase
     * @param $aParameter_chld array parameter (type row tva_belge.parameter_child)
     * @return float : amount
     * @see \Belgian\DB\Parameter_Chld_SQL
     */
    private function get_vat_filter_autoreverse($aParameter_chld)
    {
        if (trim($aParameter_chld['tva_id']) == "") return 0;

        // $pcm_val string to filter on accounting
        $pcm_val = (\noalyss_trim($aParameter_chld['pcm_val']) == "") ? "%" : $aParameter_chld['pcm_val'];

        // $a_sql_param array for SQL
        $a_sql_param = array(
            $aParameter_chld['tva_id'],
            $this->start_periode,
            $this->end_periode,
            $pcm_val
        );
        // variable: $flag_ledger to determine what formula to use to compute VAT, from PURCHASE or SALES
        $flag_ledger=$aParameter_chld['filter_ledger'] ;

        // variable: $filter_ledger string to filter on a specific ledger
        $filter_ledger = "";

        if ($aParameter_chld['filter_ledger'] == 3 && $aParameter_chld['ledger_id'] != "") {
            $filter_ledger = ' and j_jrn_def=$5';
            $a_sql_param[] = $aParameter_chld['ledger_id'];
            $x=$this->db->get_value('select jrn_def_type from jrn_def where jrn_def_id=$1',
                [$aParameter_chld['ledger_id']]);
            $flag_ledger=match($x){'VEN'=>1,'ACH'=>2,default=>0};

        }

        if ($flag_ledger == 0)
            throw new \Exception("TVMNT276. Journal invalid pour jrn_def_id{$aParameter_chld['ledger_id']}");

        // from sales
        if ($flag_ledger == 1) {
            $sql = "select coalesce(sum(qs_vat_sided),0) as amount 
            from 
                quant_sold 
                join jrnx using (j_id)
           where 
               qs_vat_code=$1
                and (j_date >= to_date($2,'DD.MM.YYYY')
                and j_date <= to_date($3,'DD.MM.YYYY'))
                and j_poste::text like ($4)
                $filter_ledger
                ";
            $res = $this->db->get_array($sql, $a_sql_param);

            return $res[0]['amount'];
        }

        // from purchase
        if ($flag_ledger  == 2) {
            $sql = "select 
                coalesce(sum(qp_vat_sided),0) as amount 
            from 
                quant_purchase 
                join jrnx using (j_id)
           where 
               qp_vat_code=$1
               and  (j_date >= to_date($2,'DD.MM.YYYY')
               and j_date <= to_date($3,'DD.MM.YYYY'))
               and j_poste::text like ($4)
               $filter_ledger
               ";
            $res = $this->db->get_array($sql, $a_sql_param);
            return $res[0]['amount'];
        }


        return 0;
    }

}

/* test::test_me(); */
